@@ -9,7 +9,7 @@ use warnings;
 
 use Scalar::Util;
 use Digest::MD5;
-our $VERSION = '0.15';
+our $VERSION = '0.16';
 
 our @EXPORT = qw[ wrap_hash ];
 
@@ -285,6 +285,8 @@ END
     my $class_template = <<'END';
 package <<CLASS>>;
 
+<<CLOSURES>>
+
 use Scalar::Util ();
 
 our $meta = { <<META>> };
@@ -474,6 +476,7 @@ sub _build_constructor {
     #<<< no tidy
     my $code = q[
     package <<PACKAGE>>;
+    <<CLOSURES>>
     <<USE>>
     use Scalar::Util ();
 
@@ -495,7 +498,7 @@ sub _build_constructor {
     ];
     #>>>
 
-    my $result = _compile_from_tpl( \$code, \%dict )
+    my $result = _compile_from_tpl( \$code, \%dict, { '$clone' => $clone }  )
       || _croak(
         "error generating constructor (as $name) subroutine: $@\n$code" );
 
@@ -519,9 +522,12 @@ sub _line_number_code {
 }
 
 
-# can't handle closures; should use Sub::Quote
 sub _compile_from_tpl {
-    my ( $code, $dict ) = @_;
+    my ( $code, $dict, $closures ) = @_;
+
+    if ( defined $closures ) {
+        $dict->{closures} = join( "\n", map { "my $_ = \$closures->{'$_'};" } keys %$closures ) ;
+    }
 
     _interpolate( $code, $dict );
 
@@ -531,6 +537,7 @@ sub _compile_from_tpl {
         print STDERR $code;
     }
 
+
     eval( $$code );    ## no critic (ProhibitStringyEval)
 }
 
@@ -588,7 +595,7 @@ Hash::Wrap - create on-the-fly objects from hashes
 
 =head1 VERSION
 
-version 0.15
+version 0.16
 
 =head1 SYNOPSIS
 
